@@ -17,6 +17,7 @@
     ArrayPrototypeFind,
     ArrayBufferIsView,
     ArrayPrototypeIncludes,
+    BigInt64Array,
     StringPrototypeToUpperCase,
     Symbol,
     SymbolFor,
@@ -37,6 +38,16 @@
 
   // P-521 is not yet supported.
   const supportedNamedCurves = ["P-256", "P-384"];
+  const recognisedUsages = [
+    "encrypt",
+    "decrypt",
+    "sign",
+    "verify",
+    "deriveKey",
+    "deriveBits",
+    "wrapKey",
+    "unwrapKey",
+  ];
 
   const simpleAlgorithmDictionaries = {
     RsaHashedKeyGenParams: { hash: "HashAlgorithmIdentifier" },
@@ -44,6 +55,7 @@
     HmacKeyGenParams: { hash: "HashAlgorithmIdentifier" },
     RsaPssParams: {},
     EcdsaParams: { hash: "HashAlgorithmIdentifier" },
+    HmacImportParams: { hash: "HashAlgorithmIdentifier" },
   };
 
   const supportedAlgorithms = {
@@ -68,6 +80,10 @@
     "verify": {
       "RSASSA-PKCS1-v1_5": null,
       "RSA-PSS": "RsaPssParams",
+      "HMAC": null,
+    },
+    "importKey": {
+      "HMAC": "HmacImportParams",
     },
   };
 
@@ -187,7 +203,7 @@
       return this[_algorithm];
     }
 
-    get [Symbol.toStringTag]() {
+    get [SymbolToStringTag]() {
       return "CryptoKey";
     }
 
@@ -224,14 +240,13 @@
   }
 
   // https://w3c.github.io/webcrypto/#concept-usage-intersection
-  // TODO(littledivy): When the need arises, make `b` a list.
   /**
    * @param {string[]} a
-   * @param {string} b
+   * @param {string[]} b
    * @returns
    */
   function usageIntersection(a, b) {
-    return ArrayPrototypeIncludes(a, b) ? [b] : [];
+    return a.filter((i) => b.includes(i));
   }
 
   // TODO(lucacasonato): this should be moved to rust
@@ -414,6 +429,166 @@
       throw new TypeError("unreachable");
     }
 
+    /**
+     * @param {string} format
+     * @param {BufferSource} keyData
+     * @param {string} algorithm
+     * @param {boolean} extractable
+     * @param {KeyUsages[]} keyUsages
+     * @returns {Promise<any>}
+     */
+    // deno-lint-ignore require-await
+    async importKey(format, keyData, algorithm, extractable, keyUsages) {
+      webidl.assertBranded(this, SubtleCrypto);
+      const prefix = "Failed to execute 'importKey' on 'SubtleCrypto'";
+      webidl.requiredArguments(arguments.length, 4, { prefix });
+      format = webidl.converters.KeyFormat(format, {
+        prefix,
+        context: "Argument 1",
+      });
+      keyData = webidl.converters.BufferSource(keyData, {
+        prefix,
+        context: "Argument 2",
+      });
+      algorithm = webidl.converters.AlgorithmIdentifier(algorithm, {
+        prefix,
+        context: "Argument 3",
+      });
+      extractable = webidl.converters.boolean(extractable, {
+        prefix,
+        context: "Argument 4",
+      });
+      keyUsages = webidl.converters["sequence<KeyUsage>"](keyUsages, {
+        prefix,
+        context: "Argument 5",
+      });
+
+      const normalizedAlgorithm = normalizeAlgorithm(algorithm, "importKey");
+
+      if (
+        ArrayPrototypeFind(
+          keyUsages,
+          (u) => !ArrayPrototypeIncludes(["sign", "verify"], u),
+        ) !== undefined
+      ) {
+        throw new DOMException("Invalid key usages", "SyntaxError");
+      }
+
+      switch (normalizedAlgorithm.name) {
+        // https://w3c.github.io/webcrypto/#hmac-operations
+        case "HMAC": {
+          switch (format) {
+            case "raw": {
+              const hash = normalizedAlgorithm.hash;
+              // 5.
+              let length = keyData.byteLength * 8;
+              // 6.
+              if (length === 0) {
+                throw new DOMException("Key length is zero", "DataError");
+              }
+              if (normalizeAlgorithm.length) {
+                // 7.
+                if (
+                  normalizedAlgorithm.length > length ||
+                  normalizedAlgorithm.length <= (length - 8)
+                ) {
+                  throw new DOMException(
+                    "Key length is invalid",
+                    "DataError",
+                  );
+                }
+                length = normalizeAlgorithm.length;
+              }
+
+              if (keyUsages.length == 0) {
+                throw new DOMException("Key usage is empty", "SyntaxError");
+              }
+
+              const handle = {};
+              WeakMapPrototypeSet(KEY_STORE, handle, {
+                type: "raw",
+                data: keyData,
+              });
+
+              const algorithm = {
+                name: "HMAC",
+                length,
+                hash,
+              };
+
+              const key = constructKey(
+                "secret",
+                true,
+                usageIntersection(keyUsages, recognisedUsages),
+                algorithm,
+                handle,
+              );
+
+              return key;
+            }
+            // TODO(@littledivy): jwk
+            default:
+              throw new DOMException("Not implemented", "NotSupportedError");
+          }
+        }
+        // TODO(@littledivy): RSASSA-PKCS1-v1_5
+        // TODO(@littledivy): RSA-PSS
+        // TODO(@littledivy): ECDSA
+        default:
+          throw new DOMException("Not implemented", "NotSupportedError");
+      }
+    }
+
+    /**
+    * @param {string} format
+    * @param {CryptoKey} key
+    * @returns {Promise<any>}
+    */
+    // deno-lint-ignore require-await
+    async exportKey(format, key) {
+      webidl.assertBranded(this, SubtleCrypto);
+      const prefix = "Failed to execute 'exportKey' on 'SubtleCrypto'";
+      webidl.requiredArguments(arguments.length, 2, { prefix });
+      format = webidl.converters.KeyFormat(format, {
+        prefix,
+        context: "Argument 1",
+      });
+      key = webidl.converters.CryptoKey(key, {
+        prefix,
+        context: "Argument 2",
+      });
+
+      const handle = key[_handle];
+      // 2.
+      const bits = WeakMapPrototypeGet(KEY_STORE, handle);
+
+      switch (key[_algorithm].name) {
+        case "HMAC": {
+          if (bits == null) {
+            throw new DOMException("Key is not available", "OperationError");
+          }
+          switch (format) {
+            // 3.
+            case "raw": {
+              for (let _i = 7 & (8 - bits.length % 8); _i > 0; _i--) {
+                bits.push(0);
+              }
+              // 4-5.
+              return bits.buffer;
+            }
+            // TODO(@littledivy): jwk
+            default:
+              throw new DOMException("Not implemented", "NotSupportedError");
+          }
+        }
+        // TODO(@littledivy): RSASSA-PKCS1-v1_5
+        // TODO(@littledivy): RSA-PSS
+        // TODO(@littledivy): ECDSA
+        default:
+          throw new DOMException("Not implemented", "NotSupportedError");
+      }
+    }
+
     /**
      * @param {string} algorithm
      * @param {CryptoKey} key
@@ -443,7 +618,7 @@
       });
 
       // 2.
-      if (ArrayBuffer.isView(signature)) {
+      if (ArrayBufferIsView(signature)) {
         signature = new Uint8Array(
           signature.buffer,
           signature.byteOffset,
@@ -452,20 +627,20 @@
       } else {
         signature = new Uint8Array(signature);
       }
-      signature = signature.slice();
+      signature = TypedArrayPrototypeSlice(signature);
 
       // 3.
-      if (ArrayBuffer.isView(data)) {
+      if (ArrayBufferIsView(data)) {
         data = new Uint8Array(data.buffer, data.byteOffset, data.byteLength);
       } else {
         data = new Uint8Array(data);
       }
-      data = data.slice();
+      data = TypedArrayPrototypeSlice(data);
 
       const normalizedAlgorithm = normalizeAlgorithm(algorithm, "verify");
 
       const handle = key[_handle];
-      const keyData = KEY_STORE.get(handle);
+      const keyData = WeakMapPrototypeGet(KEY_STORE, handle);
 
       if (normalizedAlgorithm.name !== key[_algorithm].name) {
         throw new DOMException(
@@ -474,7 +649,7 @@
         );
       }
 
-      if (!key[_usages].includes("verify")) {
+      if (!ArrayPrototypeIncludes(key[_usages], "verify")) {
         throw new DOMException(
           "Key does not support the 'verify' operation.",
           "InvalidAccessError",
@@ -516,6 +691,15 @@
             signature,
           }, data);
         }
+        case "HMAC": {
+          const hash = key[_algorithm].hash.name;
+          return await core.opAsync("op_crypto_verify_key", {
+            key: keyData,
+            algorithm: "HMAC",
+            hash,
+            signature,
+          }, data);
+        }
       }
 
       throw new TypeError("unreachable");
@@ -622,7 +806,7 @@
         const publicKey = constructKey(
           "public",
           true,
-          usageIntersection(usages, "verify"),
+          usageIntersection(usages, ["verify"]),
           algorithm,
           handle,
         );
@@ -631,7 +815,7 @@
         const privateKey = constructKey(
           "private",
           extractable,
-          usageIntersection(usages, "sign"),
+          usageIntersection(usages, ["sign"]),
           algorithm,
           handle,
         );
@@ -681,7 +865,7 @@
         const publicKey = constructKey(
           "public",
           true,
-          usageIntersection(usages, "verify"),
+          usageIntersection(usages, ["verify"]),
           algorithm,
           handle,
         );
@@ -690,7 +874,7 @@
         const privateKey = constructKey(
           "private",
           extractable,
-          usageIntersection(usages, "sign"),
+          usageIntersection(usages, ["sign"]),
           algorithm,
           handle,
         );
@@ -776,11 +960,13 @@
         !(
           arrayBufferView instanceof Int8Array ||
           arrayBufferView instanceof Uint8Array ||
+          arrayBufferView instanceof Uint8ClampedArray ||
           arrayBufferView instanceof Int16Array ||
           arrayBufferView instanceof Uint16Array ||
           arrayBufferView instanceof Int32Array ||
           arrayBufferView instanceof Uint32Array ||
-          arrayBufferView instanceof Uint8ClampedArray
+          arrayBufferView instanceof BigInt64Array ||
+          arrayBufferView instanceof BigUint64Array
         )
       ) {
         throw new DOMException(
